==> v1.9.x.zip: opentelemetry-java/sdk/metrics/src/test/java/io/opentelemetry/sdk/metrics/internal/state/DeltaMetricStorageTest.java <==
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */

package io.opentelemetry.sdk.metrics.internal.state;

import static io.opentelemetry.sdk.testing.assertj.OpenTelemetryAssertions.assertThat;

import io.opentelemetry.api.common.Attributes;
import io.opentelemetry.context.Context;
import io.opentelemetry.sdk.metrics.common.InstrumentType;
import io.opentelemetry.sdk.metrics.common.InstrumentValueType;
import io.opentelemetry.sdk.metrics.exemplar.ExemplarFilter;
import io.opentelemetry.sdk.metrics.internal.aggregator.DoubleAccumulation;
import io.opentelemetry.sdk.metrics.internal.descriptor.InstrumentDescriptor;
import io.opentelemetry.sdk.metrics.internal.export.CollectionHandle;
import io.opentelemetry.sdk.metrics.view.Aggregation;
import java.util.Set;
import java.util.function.Supplier;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class DeltaMetricStorageTest {
  private static final InstrumentDescriptor DESCRIPTOR =
      InstrumentDescriptor.create(
          "name", "description", "unit", InstrumentType.COUNTER, InstrumentValueType.DOUBLE);

  private CollectionHandle collector1;
  private CollectionHandle collector2;
  private Set<CollectionHandle> allCollectors;
  private DeltaMetricStorage<DoubleAccumulation> storage;

  @BeforeEach
  void setup() {
    Supplier<CollectionHandle> supplier = CollectionHandle.createSupplier();
    collector1 = supplier.get();
    collector2 = supplier.get();
    allCollectors = CollectionHandle.mutableSet();
    allCollectors.add(collector1);
    allCollectors.add(collector2);
    storage =
        new DeltaMetricStorage<>(
            Aggregation.sum().createAggregator(DESCRIPTOR, ExemplarFilter.neverSample()),
            DESCRIPTOR);
  }

  @Test
  void collectionDeltaForMultiReader() {
    BoundStorageHandle bound = storage.bind(Attributes.empty());
    bound.recordDouble(1, Attributes.empty(), Context.root());
    // First collector only sees first recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));

    bound.recordDouble(2, Attributes.empty(), Context.root());
    // First collector only sees second recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(2));

    // First collector no longer sees a recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .isEmpty();

    // Second collector gets merged recordings
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(3));

    // Second collector no longer sees a recording.
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ false))
        .isEmpty();
  }

  @Test
  void avoidCollectionInRapidSuccession() {
    BoundStorageHandle bound = storage.bind(Attributes.empty());
    bound.recordDouble(1, Attributes.empty(), Context.root());
    // First collector only sees first recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));
    // Add some data immediately after read, but pretent it hasn't been long.
    bound.recordDouble(2, Attributes.empty(), Context.root());
    // Collector1 doesn't see new data, because we don't recollect, but collector2 sees old delta.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ true))
        .isEmpty();
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ true))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));
    // After enough time passes, collector1 sees new data
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(2));
  }
}

==> v1.14.x.zip: opentelemetry-java/sdk/metrics/src/test/java/io/opentelemetry/sdk/metrics/internal/state/DeltaMetricStorageTest.java <==
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */

package io.opentelemetry.sdk.metrics.internal.state;

import static io.opentelemetry.sdk.testing.assertj.OpenTelemetryAssertions.assertThat;

import io.opentelemetry.api.common.Attributes;
import io.opentelemetry.context.Context;
import io.opentelemetry.sdk.metrics.Aggregation;
import io.opentelemetry.sdk.metrics.InstrumentType;
import io.opentelemetry.sdk.metrics.InstrumentValueType;
import io.opentelemetry.sdk.metrics.data.DoubleExemplarData;
import io.opentelemetry.sdk.metrics.internal.aggregator.AggregatorFactory;
import io.opentelemetry.sdk.metrics.internal.aggregator.DoubleAccumulation;
import io.opentelemetry.sdk.metrics.internal.descriptor.InstrumentDescriptor;
import io.opentelemetry.sdk.metrics.internal.exemplar.ExemplarFilter;
import io.opentelemetry.sdk.metrics.internal.export.CollectionHandle;
import java.util.Set;
import java.util.function.Supplier;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class DeltaMetricStorageTest {
  private static final InstrumentDescriptor DESCRIPTOR =
      InstrumentDescriptor.create(
          "name", "description", "unit", InstrumentType.COUNTER, InstrumentValueType.DOUBLE);

  private CollectionHandle collector1;
  private CollectionHandle collector2;
  private Set<CollectionHandle> allCollectors;
  private DeltaMetricStorage<DoubleAccumulation, DoubleExemplarData> storage;

  @BeforeEach
  void setup() {
    Supplier<CollectionHandle> supplier = CollectionHandle.createSupplier();
    collector1 = supplier.get();
    collector2 = supplier.get();
    allCollectors = CollectionHandle.mutableSet();
    allCollectors.add(collector1);
    allCollectors.add(collector2);
    storage =
        new DeltaMetricStorage<>(
            ((AggregatorFactory) Aggregation.sum())
                .createAggregator(DESCRIPTOR, ExemplarFilter.neverSample()),
            DESCRIPTOR);
  }

  @Test
  void collectionDeltaForMultiReader() {
    BoundStorageHandle bound = storage.bind(Attributes.empty());
    bound.recordDouble(1, Attributes.empty(), Context.root());
    // First collector only sees first recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));

    bound.recordDouble(2, Attributes.empty(), Context.root());
    // First collector only sees second recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(2));

    // First collector no longer sees a recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .isEmpty();

    // Second collector gets merged recordings
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(3));

    // Second collector no longer sees a recording.
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ false))
        .isEmpty();
  }

  @Test
  void avoidCollectionInRapidSuccession() {
    BoundStorageHandle bound = storage.bind(Attributes.empty());
    bound.recordDouble(1, Attributes.empty(), Context.root());
    // First collector only sees first recording.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));
    // Add some data immediately after read, but pretent it hasn't been long.
    bound.recordDouble(2, Attributes.empty(), Context.root());
    // Collector1 doesn't see new data, because we don't recollect, but collector2 sees old delta.
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ true))
        .isEmpty();
    assertThat(storage.collectFor(collector2, allCollectors, /* suppressCollection=*/ true))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(1));
    // After enough time passes, collector1 sees new data
    assertThat(storage.collectFor(collector1, allCollectors, /* suppressCollection=*/ false))
        .hasSize(1)
        .hasEntrySatisfying(Attributes.empty(), value -> assertThat(value.getValue()).isEqualTo(2));
  }
}
